package djmil.cordacheckers.gameproposal;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.contracts.GameProposalContract;
import djmil.cordacheckers.states.GameProposalState;
import net.corda.v5.application.flows.ClientRequestBody;
import net.corda.v5.application.flows.ClientStartableFlow;
import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.flows.FlowEngine;
import net.corda.v5.application.flows.InitiatingFlow;
import net.corda.v5.application.marshalling.JsonMarshallingService;
import net.corda.v5.application.membership.MemberLookup;
import net.corda.v5.application.messaging.FlowMessaging;
import net.corda.v5.application.messaging.FlowSession;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.base.exceptions.CordaRuntimeException;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.common.NotaryLookup;
import net.corda.v5.ledger.utxo.UtxoLedgerService;
import net.corda.v5.ledger.utxo.transaction.UtxoSignedTransaction;
import net.corda.v5.ledger.utxo.transaction.UtxoTransactionBuilder;
import net.corda.v5.membership.MemberInfo;
import net.corda.v5.membership.NotaryInfo;

import static java.util.Objects.requireNonNull;

import java.time.Duration;
import java.time.Instant;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

@InitiatingFlow(protocol = "game-proposal-create")
public class CreateFlow implements ClientStartableFlow{

     private final static Logger log = LoggerFactory.getLogger(CreateFlow.class);

     @CordaInject
     public JsonMarshallingService jsonMarshallingService;

     @CordaInject
     public MemberLookup memberLookup;

     @CordaInject
     public NotaryLookup notaryLookup;

     @CordaInject
     public UtxoLedgerService ledgerService;

     @CordaInject
     public FlowEngine flowEngine;

     @CordaInject
     public FlowMessaging flowMessaging;

     @Suspendable
     @Override
     public String call(ClientRequestBody requestBody) {
          try {
               log.info("flow: Create Game Proposal");

               GameProposalState gameProposal = buildGameProposalStateFrom(requestBody);
               String result = doTrunsaction(gameProposal);

               return result;
          } 
          catch (Exception e) {
               log.warn("CreateGameProposal flow failed to process utxo request body " + requestBody + " because: " + e.getMessage());
               throw new CordaRuntimeException(e.getMessage());
          }
     }

     @Suspendable
     private GameProposalState buildGameProposalStateFrom(ClientRequestBody requestBody) {
          CreateFlowArgs args = requestBody.getRequestBodyAs(jsonMarshallingService, CreateFlowArgs.class);

          GameProposalState.Color opponentColor = GameProposalState.Color.valueOf(args.opponentColor);
          if (opponentColor == null) {
               throw new RuntimeException("Allowed values for opponentColor are: "
                    + GameProposalState.Color.WHITE.name() +", "  + GameProposalState.Color.BLACK.name() 
                    + ". Actual value was: " + args.opponentColor);
          }

          MemberInfo myInfo = memberLookup.myInfo();
          MemberInfo opponentInfo = requireNonNull(
               memberLookup.lookup(MemberX500Name.parse(args.opponentName)),
               "MemberLookup can't find opponentName specified in flow arguments: " + args.opponentName
          );

          return new GameProposalState(
               myInfo.getName(),
               opponentInfo.getName(),
               GameProposalState.Color.valueOf(args.opponentColor),
               args.additionalMessage,
               UUID.randomUUID(),
               Arrays.asList(myInfo.getLedgerKeys().get(0), opponentInfo.getLedgerKeys().get(0))
          );
     }

     @Suspendable
     private String doTrunsaction(GameProposalState gameProposal) {     
          NotaryInfo notary = notaryLookup.getNotaryServices().iterator().next();

          UtxoTransactionBuilder txBuilder = ledgerService.createTransactionBuilder()
               .setNotary(notary.getName())
               .setTimeWindowBetween(Instant.now(), Instant.now().plusMillis(Duration.ofDays(1).toMillis()))
               .addOutputState(gameProposal)
               .addCommand(new GameProposalContract.Create())
               .addSignatories(gameProposal.getParticipants());

          UtxoSignedTransaction signedTransaction = txBuilder.toSignedTransaction();

          FlowSession session = flowMessaging.initiateFlow(gameProposal.getRecipient());

          List<FlowSession> sessionsList = Arrays.asList(session);

          UtxoSignedTransaction finalizedSignedTransaction = ledgerService
               .finalize(signedTransaction, sessionsList)
               .getTransaction();

          return  finalizedSignedTransaction.getId().toString();
     }
}
