package djmil.cordacheckers.gameproposal;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.states.GameProposalState;
import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.flows.InitiatedBy;
import net.corda.v5.application.flows.ResponderFlow;
import net.corda.v5.application.membership.MemberLookup;
import net.corda.v5.application.messaging.FlowSession;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.base.exceptions.CordaRuntimeException;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.UtxoLedgerService;
import net.corda.v5.ledger.utxo.transaction.UtxoSignedTransaction;
import net.corda.v5.ledger.utxo.transaction.UtxoTransactionValidator;

@InitiatedBy(protocol = "game-proposal-create")
public class CreateResponder implements ResponderFlow {
    private final static Logger log = LoggerFactory.getLogger(CreateResponder.class);

    @CordaInject
    public MemberLookup memberLookup;

    @CordaInject
    public UtxoLedgerService utxoLedgerService;
    
    @Suspendable
    @Override
    public void call(FlowSession session) {
        try {
            // Defines the lambda validator used in receiveFinality below.
            UtxoTransactionValidator txValidator = ledgerTransaction -> {
                GameProposalState gameProposal = (GameProposalState) ledgerTransaction.getOutputContractStates().get(0);
                // Uses checkForBannedWords() and checkMessageFromMatchesCounterparty() functions
                // to check whether to sign the transaction.
                if (!checkParticipants(gameProposal, session.getCounterparty())) {
                    throw new CordaRuntimeException("Failed verification");
                }

                log.info("Verified the transaction - " + ledgerTransaction.getId());
            };

            // Calls receiveFinality() function which provides the responder to the finalise() function
            // in the Initiating Flow. Accepts a lambda validator containing the business logic to decide whether
            // responder should sign the Transaction.
            UtxoSignedTransaction finalizedSignedTransaction = utxoLedgerService
                    .receiveFinality(session, txValidator)
                    .getTransaction();

            log.info("Finished responder flow - " + finalizedSignedTransaction.getId());
        }
        // Soft fails the flow and log the exception.
        catch(Exception e)
        {
            log.warn("Exceptionally finished responder flow", e);
        }
    }

    @Suspendable
    Boolean checkParticipants(GameProposalState gameProposal, MemberX500Name counterpartyName) {
        MemberX500Name myName = memberLookup.myInfo().getName();

        if (gameProposal.getRecipient().compareTo(myName) == 0 && 
            gameProposal.getSender().compareTo(counterpartyName) == 0)
            return true;
    
        return false;
    }
    
}
